/**
 * 手机版（app) 通用JS
 * 常用的函数方法
 */
var isAndroid = (/android/gi).test(navigator.appVersion),

    // 检查是否支持 滑动
    hasTouch = 'ontouchstart' in window,
    
    // Events
	RESIZE_EVENT = 'onorientationchange' in window ? 'orientationchange' : 'resize',
    UNLOAD_EVENT = 'onpagehide' in window ? 'pagehide' : 'beforeunload';
	START_EVENT = hasTouch ? 'touchstart' : 'mousedown',
	MOVE_EVENT = hasTouch ? 'touchmove' : 'mousemove',
	END_EVENT = hasTouch ? 'touchend' : 'mouseup',
    CLICK_EVENT = hasTouch ? 'tap' : 'click';

// 如果API提供了存储对象，就用API的
var dataStorage = window.dataStorage || window.localStorage;
var dataSession = window.sessionStorage || {
    getItem: function() {},
    setItem: function() {}, 
    removeItem: function() {}
};

// 数据API
var msxfAPI = window.msxfAPI || {
    loadText: function(bid, aid) { return "nodata"; }, 
    onLine:function() {
        return true;
    },
    goUrl: function(url) {
        if (url) window.location.href = url;
    },
    getNetworkType: function() {
        return "WIFI";
    },
    getChannel: function() { return "msxf"; },
    mobileUniqid: function () {
        return $.uniqid()
    },
    mobileNumber: function() {},
    showMsg: function(msg) {
        $.tip().show(msg);
    },
    debug: function(msg) {
        console.log(msg);
    },
    readBuyRecord: function(bid, aid) {},
    saveBuyRecord: function(bid, aid, buymode) {},
    existChapter: function(bid, aindex) { return false; },
    saveChapter: function(bid, aindex, data) {},
    existBook: function(bid) { return false; },
    downApk: function(url, name) {},
    downZip: function(url, name) {},
    allowOffline: function() { return false; } 
};
// 作品API
var bookAPI = window.bookAPI || {
    getBookList: function() {},
    getBookInfo: function(bid) {},
    removeBook: function(bid) {},
    existBook: function(bid) { return false; }
};
// 文件系统API
var fileAPI = window.fileAPI || {
    defRoot: function() {
        var defaultData = '{"sd_root": "/mnt/sdcard", "sd_ext_root": "B", "app_root": "/mnt/sdcard/msxfapp/file"}';
        return defaultData;
    },
    chapter: function(path, page, length) {
        return '{"ctotal": 10, "content": "test chapter"}';
    },
    removeBook: function(id) {},
    bookList: function() {
        // 已经加入书架的数据列表
        return '[{"path": "a", "name": "bbbb", "id": 1}, {"path": "a", "name": "ccbbbb", id: 2}]';
    },
    fileList: function() {
        return '{"prev_dir": "/ABC", "dir": [{"name": "test", "count": 123, "dir": "abcdefg"}], "files": [{"is_join": 1,"name": "test", "size": "24M", "date": "2014-10-11", "dir": "abcdefg"}, {"name": "test2", "size": "14M", "date": "2014-10-12", "dir": "abcdefg"}]}';
    },
    joinShelf: function(name, path) {
        showMsg("name:"+name +"|"+"path:"+path);
    }
};

// 系统配置参数前缀
var systemConfigPrefix = "system.config.";

var domain = "http://phonedev.msxf.net/";
// 主机
function server(host) {
    var url = "http://api.msxf.net";
    if (host == "apk") {
         url = "http://api.msxf.net/apk.php";
    }
    return url;
}

// 调试信息
function debug(msg) {
    // 输出调试信息
    msxfAPI.debug(msg);
}

// 显示提示信息
function showMsg(msg) {
    msxfAPI.showMsg(msg);
}

// 日期比较 YYYY-mm-dd 
function dateDiff(startTime, endTime) {
    var dateStartTime = new Date(startTime.replace(/-/g,"/"));   
    var dateEndTime = new Date(endTime.replace(/-/g,"/"));  

    var days = (dateEndTime.getTime() - dateStartTime.getTime()) / (1000 * 86400);

    return days;
}

// 返回当前的 YYYY/M/D H:i:s
function datetime() {
    var t = new Date();
    return t.getFullYear() + '/' + parseInt(t.getMonth())+1 + '/' + t.getDate() + ' ' + t.getHours() + ':' + t.getMinutes() + ':' + t.getSeconds();
}

function time() {
    return Math.round(new Date().getTime() / 1000);
}

// 格式化 将 {0},{1} 这样的参数进行替换
function format(b) {
    if (arguments.length == 0) {
        return "";
    }
    var a = Array.prototype.slice.call(arguments, 1);
    return b.replace(/\{[\w]*\}/g, function(c) {
        return a[c.replace(/^\{|\}$/g, "")]
    })
}

// 检测是否为空
function nl(a) {
    return a == null || a == "";
}

// 三元运算，是否为空
function nlg(b, a) {
    return nl(b) ? a : b;
}

// 字符串长度 按utf8的算
function mb_strlen(str) {
	var len = 0;
	for (var i = 0; i < str.length; i++) {
		len += str.charCodeAt(i) < 0 || str.charCodeAt(i) > 255 ? 3 : 1;
	}
	return len;
}

// 模版解析方法 {{key}}
function templateToData(template, data) {
    if (!data) { return null; }

    var i = 0,
        len = data.length,
        fragment = '';

    function replace(obj) {
        var t, key, reg;
　　　　　　
        for (key in obj) {
            reg = new RegExp('{{' + key + '}}', 'ig');
            t = (t || template).replace(reg, obj[key]);
        }

        return t;
    }

    for (; i < len; i++) {
        fragment += replace(data[i]);
    }

    return fragment;
}

// 判断值是否在数组里面
function inArray(needle,array, bool) {
	if (typeof needle == "string" || typeof needle == "number") {
	    var len=array.length;
		for (var i = 0; i< len; i++) {
			if (needle === array[i]) {
				if (bool) {
					return i;
				}
				return true;
			}
		}
		return false;
	}
}

// 检测对象是否为元或DOC对象
function isElementOrDocument(obj) {
    return obj.nodeType == 1 || obj.nodeType == 9;
}

// 自定义的下拉菜单
function showDropDown(showid, defaultIndex, moreCallback) {

    // 当前显示的对象，和下拉对象
    var showObj = $("#"+showid),
        moreObj = $("#"+showid+"_more");

    showObj.bind("click", function(e) {
        e.preventDefault();
        // 显示菜单
        moreObj.toggleClass("show");
    });

    // 选择类型
    moreObj.bind("click", function(e) {
        if (!e) return false;
        var target = e.target;

        var data_value = target.attr("data-value"),
            html = target.html();
        
        showObj.attr({"data-value": data_value}).html(html);
        // 隐藏下拉菜单
        moreObj.removeClass("show");
        
        // 点击菜单，回调事件
        if (moreCallback && typeof moreCallback == 'function') {
			moreCallback(data_value);
		}
    });

    // 设置默认选中的
    if (defaultIndex) {
        var defaultObj = moreObj.$s("p");
        if (defaultObj && defaultObj.length > defaultIndex - 1) {
            var target = defaultObj[defaultIndex-1];
            if (target) {
                var data_value = target.attr("data-value"),
                    html = target.html();
                
                showObj.attr({"data-value": data_value}).html(html);
            }
        }
    }
}

// 关闭自定义下拉菜单
function hideDropDown(showid, target) {
    if (!target || target.id != showid) {
        $("#"+showid+"_more").removeClass("show");
    }
}

// 给页面增加输入法高度
function imeHeight(height) {
    var height = height || 200;
    var main = $("#main");
    var curr_height = parseInt(main.css("height"));
    main.css({"height": curr_height + height});
}

// 设置或读取用作返回上页的名称 主要在几个列表页面使用
function backPagePath(ac, path) {
    var id = "book.lastpage";
    if (ac == "get") {
        return dataStorage.getItem(id);
    } else if (ac == "set") {
        if (!path) {
            // 没有设置path,就取当前文件名
            var currpath = window.location.pathname.substring(1);
            var path_split = currpath.split("/");
            if (path_split) path = path_split[path_split.length - 1];
        }
        if (path) dataStorage.setItem(id, path);
    } else if (ac == "remove") {
        dataStorage.removeItem(id);
    }
}

// 根据aindex取得对应的article_id
function indexToAid(bid, aindex) {
    var bookchapter = dataStorage.getItem("book.chapter."+bid);
    if (bookchapter) {
        try {
            chapters = JSON.parse(bookchapter);
            if (chapters[aindex - 1]) {
                return chapters[aindex-1].article_id;
            }
        } catch(e) {}
    }

    return 0;
}

// 保存或读取当前pos
function currentPosData(id, pos) {
    if (!id) return false;

    var saveId = "pos." + id;
    if (!isNaN(pos)) {
        dataSession.setItem(saveId, pos);
    } else {
        return parseInt(dataSession.getItem(saveId));
    }
}

// 清理指定前缀的缓存
function clearStorageData(prefix) {
    if (!prefix) return false;

    // 只删除 prefix 
    if (dataStorage.removeItemByPrefix) {
        dataStorage.removeItemByPrefix(prefix);
    } else {
        for (var key in dataStorage) {
            if (key.substring(0, prefix.length) == prefix) {
                dataStorage.removeItem(key);
            }
        }
    }
}

/**
 * 缓存章节目录
 * curr_count 当前已经保存的章节数量
 */
function cacheChapterDir(bid, curr_count, callback) {
    try {
        // 先判断是否有网络
        if (msxfAPI.onLine() == false) return false;

        var url = server() + '/book/dir?bid='+bid+'&curr_count='+curr_count;
        $.jsonp(url, function(ret) {
            if (!ret) return false;
            if (!ret.list_chapter) return false;
            
            // 只有章节数量更新的时候，才会重新下载保存
            if (ret.list_chapter.length > curr_count) {
                // 章节缓存
                dataStorage.setItem("book.chapter."+bid, JSON.stringify(ret.list_chapter));
                // 章节数量
                dataStorage.setItem("book.chapter_total."+bid, ret.list_chapter.length);
            }
            
            if (typeof callback == "function") callback();

            delete ret;
        }, 'xhr');
    } catch(e) {}
}

// safe code
function SAFE_CODE() {
    var safe = dataStorage.getItem("user.safe");
    if (!safe) {
        safe = msxfAPI.mobileUniqid();
        dataStorage.setItem("user.safe", safe);
    }

    return safe;
}

// 获取用户的sessid
function SID() {
    return dataStorage.getItem("user.sessid");
}

// 退出登陆
function userLoginOut() {
    dataStorage.removeItem("user.sessid");
    dataStorage.removeItem("user.info");
    dataStorage.removeItem("user.autoreg");
}

// 用户头像
function useravatar(uid, size) {
    if (!uid) return "";

    // 转换成字符串
    uid = uid.toString();

    // 先判断有没有6位，没有就在前面补0
    var minUidLength = 6 - uid.length;
    if (minUidLength > 0) {
        var placeholder = '';
        for (var i = 0; i < minUidLength; i++) {
            placeholder += "0";
        }
        uid = placeholder + uid;
    }

    var base_url = "http://passport.nuozhan.com/customavatars";
    var dir1 = uid.substr(0, 2);
    var dir2 = uid.substr(2, 2);
    var lastid = uid.substr(-2);

    var avatar_url = "";
    if (size) {
        avatar_url = base_url + "/"+dir1+"/"+dir2+"/"+lastid+"_avatar_"+size+".jpg";
    } else {
        avatar_url = base_url + "/"+dir1+"/"+dir2+"/"+lastid+"_avatar_small.jpg";
    }

    return avatar_url;
}

// 默认封面
function defaultCover() {
    return 'img/default_cover.jpg';
}

// 输出封面 progress 进度情况
function fetchBookCoverCode(bid, size, progress) {
    // 判断网络，来显示封面
    var src = defaultCover();
    if (msxfAPI.onLine() == true) {
        src = bookcover(bid, size);
    }

    var html = '<div class="book-cover">';
        html += '<img src="'+src+'" loaded="1">';
    if (progress) {
        html += progress;
    }
    html += '<span class="shadow"></span></div>';
    return html;
}

// 计算出作品封面地址
function bookcover(bid, size) {
    if (!bid) return "";
    
    var size = size || 'small';
    
    // 转换成字符串
    bid = bid.toString();

    var base_url = "http://www.msxf.net/upload";
    var dir = bid.substr(0, 2);
    var cover_url = base_url + "/small_cover/book/"+dir+"/"+bid+".jpg";
    if (size == "middle") {
        cover_url = base_url + "/middle_cover/book/"+dir+"/"+bid+".png";
    } else if (size == "big") {
        cover_url = base_url + "/book/"+dir+"/"+bid+".png";
    }

    return cover_url;
}

// 下载炫色书城在线apk
function downMsxfApk() {
    var url = 'http://www.msxf.net/public/app/msxfapp.apk';
    msxfAPI.downApk(url, "msxfapp.apk", "msxf", "msxfapp");
}

// 下载离线作品
function downBook(key, name, type) {
    if (!key) return false;
    
    var type = type || "book";
    var url = server('apk') + "/store/down?key="+key+"&type="+type;

    if (type == "zip") { // 下载zip
        msxfAPI.downZip(url, name, type, key);
    } else { // 下载apk
        msxfAPI.downApk(url, name, type, key);
    }
}

// 书架首页
function openShelf() {
    go("shelf.html");
}

// 书城首页
function openStore() {
    goUrl("start.html");
}

// 进入我的书包
function openUserPacket() {
    goUrl("user_packet.html");
}

// 打开章节目录
function openDir(aindex) {
    dataStorage.setItem("read.aindex", aindex);
    go("read_dir.html");
}

// 进入本地目录
function openLocalFile() {
    go("local_file.html");
}

// 回书页
function openBook(bid, btype) {
    dataStorage.setItem("read.bid", parseInt(bid));
    if (btype) dataStorage.setItem("read.btype", btype);

    if (btype == "down") { // 标识为下载的作品，要判断是否存在
        var exist = bookAPI.existBook(bid);
        if (!exist) {
            goUrl("book.html");
            return;
        }
    }
    go("book.html");
}

// 去阅读
function openRead(bid, aindex, page) {
    var page = page || 1;

    // 如果掉线了，要判断一下章节是否保存过，如果没有就提示
    if (msxfAPI.onLine() == false) {
        var is_chapter = msxfAPI.existChapter(bid, aindex);
        if (!is_chapter) {
            showMsg("网络已掉线，无法载入章节");
            return false;
        }
    }

    dataStorage.setItem("read.bid", parseInt(bid));
    dataStorage.setItem("read.aindex", parseInt(aindex));
    dataStorage.setItem("read.page", page);

    go("read.html");
}

// 阅读TXT
function openReadTxt(path, name, page) {
    var page = page || 0;

    dataStorage.setItem("read.txt.path", path);
    dataStorage.setItem("read.txt.name", name);
    dataStorage.setItem("read.txt.page", page);

    go("read_txt.html");
}

// 去充值
function openPay(bid) {
    backPagePath("set");
    // 记录是由哪本书进入的充值
    if (bid) {
        dataStorage.setItem("pay.bid", bid);
    }
    goUrl("pay.html");
}

// 最近阅读
function openHistory() {
    go("history.html");
}

// 在线书架
function openOnlineShelf() {
    goUrl("shelf.html");
}

// 为我推荐
function openGuess() {
    goUrl("guess.html");
}

// 获取URL里的参数值
function getQueryString(name) {
    var reg = new RegExp("(^|&)" + name + "=([^&]*)(&|$)", "i");
    var r = window.location.search.substr(1).match(reg);
    if (r != null) return unescape(r[2]); 
    return null;
}

// 转向到在线域名地址
function goUrl(href) {
    if (msxfAPI.onLine() == false) {
        showMsg("网络已掉线，请联网后重试");
        return false;
    }

    // 如果不是wifi 就显示个载入进度条
    try {
        var networkType = msxfAPI.getNetworkType();
        if (networkType && networkType != 'WIFI') {
            showMsg("正在载入中...");
        }
    } catch(e) {}

    msxfAPI.goUrl(domain + href);
}

// 转向
function go(href, online) {
    // 网络掉线了
    if (online && msxfAPI.onLine() == false) {
        showMsg("网络已掉线，请联网后重试");
        return false;
    }

    var redirect = setTimeout(function() {
        redirect = null;
        window.location.href = href;
    }, 300);
}

// 转向，新窗口
function to(target) {
    if (!target) return false;

    var ev = document.createEvent("Events");
    ev.initEvent("click", true, true);
    target.dispatchEvent(ev);
}

// 显示确认对话框
function showDialogConfirm(conf, yesBtn) {
    var msg = conf.msg || "确认要继续吗？";
    var yestxt = conf.yestxt || "确定";

    // 背景
    var wrap = document.createElement("div");
    wrap.css({"position": "absolute", "zIndex": "9998", "visibility": "visible", "backgroundColor": "#908E8F", "opacity": 0.45, "top": 0, "left": 0, "bottom": 0, "right": 0, "width": "100%", "height": "100%"});
    $("body").insertChild(wrap);

    // 显示确认框
    var dialog = document.createElement("div");
    dialog.setClass("btn_alert_bg").css({"zIndex": "9999", "position": "absolute", "bottom": 0, "left": 0, "right": 0, "-webkit-transition": "top 300ms ease-out"});

    dialog.html('<p>'+msg+'</p><p><span class="btn_color_green" data-id="yes">'+yestxt+'</span></p><p><span class="btn_color_blue" data-id="no">取消</span></p>');
    $("body").insertChild(dialog);

    dialog.bind(START_EVENT, function(e) {
        e.preventDefault();
    });
    dialog.bind(MOVE_EVENT, function(e) { 
        e.preventDefault(); 
    });
    dialog.bind(END_EVENT, function(e) {
        e.preventDefault();
        if (e.target != null) {
            var id = e.target.attr("data-id");
            if (id == "yes") {
                if (typeof yesBtn == "function") yesBtn();
                dialog.remove();
                wrap.remove();
            } else if (id == "no") {
                dialog.remove();
                wrap.remove();
            }                    
        }
    });           
    
    // 返回
    wrap.bind(START_EVENT, function(e) {
        e.preventDefault();
        dialog.remove();
        wrap.remove();
    });
}

// 显示登陆对话框
function showLoginDialog(callback) {
    var iw = window.innerWidth;
    var ih = window.innerHeight;

    var doc = document, ul, li;

    var wrapObj = $("#comm_user_wrap");
    var dialogObj = $("#comm_user_dialog");

    // 检查对话框是不是已经创建出来过了，因为$方法总是返回一个对象
    if (wrapObj.id && dialogObj.id) return;

    // 背景
    var wrap = doc.createElement("div");
    wrap.css({"position": "absolute", "zIndex": "5555", "visibility": "visible", "backgroundColor": "#908E8F", "opacity": 0.45, "top": 0, "left": 0, "bottom": 0, "right": 0, "width": "100%", "height": "100%"});
    wrap.attr({"id": "comm_user_wrap"});
    $("body").insertChild(wrap);

    // 显示确认框
    var dialog = doc.createElement("div");
    dialog.setClass("comm_user").attr({"id": "comm_user_dialog"});

    // title
    var dialog_title = doc.createElement("div");
    dialog_title.setClass("comm_dialog_title").html('用户登陆<span class="close" data-id="close"></span>');
    dialog.insertChild(dialog_title);

    // form
    var dialog_form = doc.createElement("div");
    dialog_form.setClass("form");
    dialog.insertChild(dialog_form);

    // ul/li
    ul = doc.createElement("ul");
    dialog_form.insertChild(ul);

    li = doc.createElement("li");
    li.attr({"data-focus": "username"}).html('<label>登陆名：</label><input type="text" id="username" placeholder="注册时填写的登陆名">');
    ul.insertChild(li);

    li = doc.createElement("li");
    li.attr({"data-focus": "password"}).html('<label>密码：</label><input type="text" id="password" class="password" placeholder="字母区分大小写">');
    ul.insertChild(li);
    
    li = doc.createElement("li");
    li.html('<label>&nbsp;</label><div data-id="btn-login" class="btn_color_green">登陆</div>');
    ul.insertChild(li);

    li = doc.createElement("li");
    li.html('<label>&nbsp;</label><div data-id="go-register" class="btn_color_blue">注册账号</div>');
    ul.insertChild(li);

    // tips
    var tips_html = '<p><img src="img/warning.png"> 如果没有账号，就先去免费注册一个。</p>';
    tips_html += '<p><img src="img/warning.png"> 如果忘了密码，<span data-id="go-getpasswd" class="btn_min">找回密码</span></p>';

    var dialog_tips = doc.createElement("div");
    dialog_tips.setClass("tips").html(tips_html);
    dialog_form.insertChild(dialog_tips);
    
    $("body").insertChild(dialog);

    dialog.bind(START_EVENT, function(e) {
        var target = e.target;
        // 行焦点
        var data_focus = target.attr("data-focus");
        if (data_focus) $("#"+data_focus).focus();

        if (target.tagName != 'INPUT') {
            e.preventDefault();
        }
        
        var id = target.attr("data-id");
        if (id == "go-register") {
            goUrl("user_register.html");
        } else if (id == "go-getpasswd") {
            goUrl("user_getpasswd.html");
        }
    });
    dialog.bind(MOVE_EVENT, function(e) { 
        e.preventDefault();
    });
    dialog.bind(END_EVENT, function(e) {
        e.preventDefault();
        if (e.target != null) {
            var id = e.target.attr("data-id");
            if (!id) return false;
            // 登陆
            if (id == "btn-login") {
                btnSubmit();
            } else if (id == "close") {
                dialog.remove();
                wrap.remove();
                if (typeof callback == "function") callback();
            }
        }
    });
    
    // 回车事件
    dialog.bind("enter", function(e) {
        btnSubmit();
    });
    
    // login
    function btnSubmit() {
        var username = $("#username").value;
        var password = $("#password").value;
        if (!username) {
            showMsg("用户名不能为空");
            return false;
        }

        if (!password) {
            showMsg("密码不能为空");
            return false;
        }

        // 网络掉线了
        if (msxfAPI.onLine() == false) {
            showMsg("网络已掉线，请重试");
            return false;
        }
        
        dataStorage.setItem("user.username", username);

        $.loading().init();
        $.loading().open();
        
        // 最多15秒，关闭进度条
        setTimeout(function() {
            $.loading().close();
        }, 15000);

        var url = server() + "/user/login?username="+encodeURIComponent(username)+"&password="+password+"&safe="+SAFE_CODE();
        $.jsonp(url, function(ret) {
            $.loading().close();
            if (!ret) {
                showMsg("操作失败");
                return false;
            }
            if (ret.errno != 0) {
                showMsg(ret.msg);
                return false;
            }
            // 登陆之后，内容清空
            $("#username").value = '';
            $("#password").value = '';

            dataStorage.setItem("user.sessid", ret.sessid);
            
            showMsg("登陆成功！");

            // 登陆成功之后，也只是关闭当前层
            dialog.remove();
            wrap.remove();
        }, "xhr");
    }
    
    // 返回
    wrap.bind(START_EVENT, function(e) {
        e.preventDefault();
        dialog.remove();
        wrap.remove();
    });
}

/**
 * 显示要购买章节的对话框
 * callback 取消订阅后的事件，有的页面取消之后，只能返回到书页去了
 */
function showOrderDialog(bid, aindex, callback) {
    var iw = window.innerWidth;
    var ih = window.innerHeight;

    // 类型转换一下
    bid = parseInt(bid);
    aindex = parseInt(aindex);

    // 判断是否登陆
    if (!SID()) {
        showLoginDialog();
        return false;
    }
    
    // 网络掉线了
    if (msxfAPI.onLine() == false) {
        showMsg("网络已掉线，请重试");
        return false;
    }
    
    // 保存以前购买过的记录
    function saveLastBuyRecord(aids) {
        if (aids) {
            for (var i = 0; i < aids.length; i++) {
                msxfAPI.saveBuyRecord(bid, parseInt(aids[i]), 'article');
            }
        }
    }

    var aid = indexToAid(bid, aindex);
    if (aid <= 0) {
        showMsg("章节ID错误，请重试");
        return false;
    }

    $.loading().init();
    $.loading().open();

    // 最多15秒，关闭进度条
    setTimeout(function() {
        $.loading().close();
    }, 15000);
    
    // APP渠道来源
    var APP_CHANNEL_NAME = msxfAPI.getChannel();
    // 手机唯一串
    var mobileUniqid = msxfAPI.mobileUniqid();
    var url = server('apk') + "/article/buy?bid="+bid+"&aid="+aid+"&pay_source="+APP_CHANNEL_NAME+"&sessid="+SID()+"&uniqid="+mobileUniqid+"&safe="+SAFE_CODE();
    $.jsonp(url, function(ret) {
        $.loading().close();
        if (!ret) {
            showMsg("章节读取失败，请重试");
            return false;
        }

        if (ret.errno != 0) {
            // 要先登陆
            if (ret.errno == -2) {
                showLoginDialog();
                return false;                
            }
            showMsg(ret.msg);
            return false;
        }

        // 判断是否购买过
        if (ret.isbuy == 1) {
            // 购买成功 要在APP里面，设置购买记录
            msxfAPI.saveBuyRecord(bid, parseInt(aid), ret.buymode);
            // 以前的购买记录
            if (ret.all_buyid) {
                saveLastBuyRecord(ret.all_buyid);
            }
            
            // 稍延时一下
            setTimeout(function() {
                openRead(bid, aindex);
            }, 100);
            return;
        }

        // 默认不选中快速订阅
        var fastbuytxt = '<span class="fl-r" data-id="fastbuy"><img src="img/checkbox2.png" id="fastbuy" data-value="0" class="fastbuy">快速续订</span>';

        // 没有买过， 给出订单项
        var booktxt = '';
        booktxt += '<p>作品名称：'+ret.book.name+'</p>';
        booktxt += '<p>订购章节：'+ret.article.title+'</p>';
        booktxt += '<p>章节价格：'+ret.article.coin+' 陌上币</p>';
        booktxt += '<p>全本字数：'+ret.book.abs_words+'</p>';
        booktxt += '<p>快速订阅：'+ret.book.price_offline+' 陌上币'+fastbuytxt+'</p>';

        var doc = document;
                
        // 背景
        var wrap = doc.createElement("div");
        wrap.css({"position": "absolute", "zIndex": "9998", "visibility": "visible", "backgroundColor": "#908E8F", "opacity": 0.45, "top": 0, "left": 0, "right": 0, "bottom": 0, "width": "100%", "height": "100%"});
        $("body").insertChild(wrap);

        var order = doc.createElement("div");
        order.setClass("order");

        // title
        var order_title = doc.createElement("div");
        order_title.setClass("comm_dialog_title").html('确认订阅<span class="close" data-id="close"></span>');
        order.insertChild(order_title);

        // box
        var order_box = doc.createElement("div");
        order_box.setClass("box");
        order.insertChild(order_box);

        var order_box_info = doc.createElement("div");
        order_box_info.setClass("info").html('<p>你好，'+ret.user.username+' <span data-id="go-logout" class="btn_min show_btn">更换账号</span></p><p>余额：'+ret.user.coin+' 个陌上币 <span data-id="go-pay" class="btn_min show_btn">充值</span></p>');
        order_box.insertChild(order_box_info);

        var order_box_book = doc.createElement("div");
        order_box_book.setClass("book").html(booktxt);
        order_box.insertChild(order_box_book);
                        
        // 订阅按钮
        var btn_txt = '<span class="btn_color_green" data-id="yes" data-value="article">订阅本章</span>';
            btn_txt += '&nbsp;&nbsp;&nbsp;&nbsp;<span class="btn_color_green" data-id="yes" data-value="full">快速订阅</span>';
        var btn_txt2 = '<span class="btn_color_blue" data-id="yes" data-value="article" data-buynum="10">订阅10章</span>';
            btn_txt2 += '&nbsp;&nbsp;&nbsp;&nbsp;<span class="btn_color_blue" data-id="yes" data-value="article" data-buynum="50">订阅50章</span>';

        var order_box_btn = doc.createElement("div");
        order_box_btn.setClass("btn").html(btn_txt);
        order_box.insertChild(order_box_btn);
        // btn2
        order_box_btn = doc.createElement("div");
        order_box_btn.setClass("btn").html(btn_txt2);
        order_box.insertChild(order_box_btn);

        var order_box_tips = doc.createElement("div");
        order_box_tips.setClass("tips").html('<img src="img/warning.png"> 请确认作品信息。批量购买章节可离线阅读，已订阅过的章节不会再次扣费，请放心订阅。推荐快速订阅！如陌上币不足，请先充值。');
        order_box.insertChild(order_box_tips);
       
        $("body").insertChild(order);

        order.bind(START_EVENT, function(e) {
            e.preventDefault();

            var target = e.target;
            if (!target) return false;

            var id = target.attr("id"), did = target.attr("data-id");
            if (id == "fastbuy" || did == "fastbuy") {
                var fastbuy = $("#fastbuy");
                var value = fastbuy.attr("data-value");

                if (value == 1) {
                    fastbuy.attr({"src": "img/checkbox2.png", "data-value": 0});
                    dataStorage.setItem("read.fastbuy", 0);
                } else {
                    fastbuy.attr({"src": "img/checkbox2_f.png", "data-value": 1});
                    dataStorage.setItem("read.fastbuy", 1);
                }
            }
            
            // 退出登陆
            if (did == "go-logout") {
                 order.remove();
                wrap.remove();
                userLoginOut();
                showLoginDialog();
                return;
            } else if (did == "go-pay") {
                openPay(bid);
            }
        });

        order.bind(MOVE_EVENT, function(e) { 
            e.preventDefault(); 
        });

        order.bind(END_EVENT, function(e) {
            e.preventDefault();
            if (!e.target) return false;

            var id = e.target.attr("data-id"),
                buymode = e.target.attr("data-value"),
                buynum = e.target.attr("data-buynum");
            if (id == "yes") {
                // 先判断陌上币够不够
                if (buymode == "full") {
                    if (parseInt(ret.user.coin) < parseInt(ret.book.price_offline)) {
                        showMsg("陌上币不足，请先充值");
                        return false;
                    }
                } else {
                    // 按章节购买
                    var mincoin = buynum > 0 ? 40 : 1;
                    if (parseInt(ret.user.coin) <= mincoin) {
                        showMsg("陌上币不足，请先充值");
                        return false;
                    }
                }

                // 网络掉线了
                if (msxfAPI.onLine() == false) {
                    showMsg("网络已掉线，请开启网络后再试");
                    return false;
                }

                // 判断是否勾上了快速订阅
                var fastbuy = dataStorage.getItem("read.fastbuy");
                if (!fastbuy) fastbuy = 0;

                $.loading().init();
                $.loading().open();

                // 最多15秒，关闭进度条
                setTimeout(function() {
                    $.loading().close();
                }, 15000);

                // 提交到订阅，成功就继续阅读
                var url = server('apk') + "/article/buy?bid="+bid+"&aid="+aid+"&ac=buy"+"&fastbuy="+fastbuy+"&sessid="+SID()+"&buymode="+buymode+"&uniqid="+mobileUniqid+"&buynum="+buynum+"&pay_source="+APP_CHANNEL_NAME+"&safe="+SAFE_CODE();
                $.jsonp(url, function(ret) {
                    $.loading().close();
                    if (!ret) {
                        showMsg("章节订阅失败，请重试");
                        return false;
                    }

                    if (ret.errno != 0) {
                        // 要先登陆
                        if (ret.errno == -2) {
                            showLoginDialog();
                            return false;                
                        }
                        showMsg(ret.msg);
                        return false;
                    }

                    // 继续阅读
                    if (ret.isbuy == 1) {
                        // 购买成功 要在APP里面，设置购买记录
                        msxfAPI.saveBuyRecord(bid, parseInt(aid), buymode);

                        // 以前购买过的，也一起设置购买记录
                        if (ret.all_buyid) {
                            saveLastBuyRecord(ret.all_buyid);
                        }
                        
                        // 稍延时一下
                        setTimeout(function() {
                            openRead(bid, aindex);
                        }, 100);
                    } else {
                        showMsg("订阅失败，请重试");
                        return false;
                    }
                }, 'xhr');

            } else if (id == "close") {
                order.remove();
                wrap.remove();
                if (typeof callback == "function") callback();
            }
        });

        // 返回
        wrap.bind(START_EVENT, function(e) {
            e.preventDefault();
            order.remove();
            wrap.remove();

            if (typeof noBtn == "function") noBtn();
        });

    }, 'xhr');    
}

// 顶部事件链接
function header_event() {
    $("#header").bind(START_EVENT, function(e) {
        e.preventDefault();
        if (e.target == null) return false;        

        // 如果是自定义处理按钮事件，就不操作
        var custom = e.target.attr("data-custom");
        if (custom) { return; }

        var id = e.target.attr("data-id");
        if (id) {
            // 书城要检查网络
            if (id == "store") {
                openStore();
            } else if (id == "back") {
                // 是否主动设置了返回页面
                var backpath = backPagePath("get");
                if (backpath) {
                    ref = backpath;
                    // 读取过后，就删掉，防止多次使用
                    backPagePath("remove");
                } else {
                    ref = "shelf.html";
                }
                go(ref);
            } else {
                // user 特殊判断
                if (id == "user" && SID()) {
                    id = "user_info";
                }
                go(id+".html");
            }
        }
    });
}
header_event();
