/**
 * MACO.JS 框架的app专用版，精简，快速
 * 
 * elemtns 元素相关的对象封装
 * array 数组
 * string 字符串
 * events 事件类
 *
 */

(function(window) {

function MACO() {

    // 当前对象
    var that = this, 
        doc = window.document,
        // 对象选择匹配规则
        rQuery = /(?:^([#\.]?)([\w-]*?))(?:\[([\w-]+)(?:=[\'\"]([^\'\"]+)[\'\"])?\])?(?:\.([\w-]+))?$/,
        MACO = function(q) { return that.$(q); };

    // 缓存的对象
    this.elexts = {};

    // elements 元素对象
    this.elements = {
        // 添加事件
        bind: function(type, fn, bubble) {
            if (type == 'enter') {
                this.addEventListener('keyup', function(e) {
                    if (e.keyCode == 13) {
                        fn.call(that);
                    }
                }, !!bubble);
            } else {
                this.addEventListener(type, fn, !!bubble);
            }

            return this;
        },

        // 删除事件
        unbind: function(type, fn, bubble) {
            if (type == 'enter') {
                this.removeEventListener('keyup', function(e) {
                    if (e.keyCode == 13) {
                        fn.call(that);
                    }
                }, !!bubble);
            } else {
                this.removeEventListener(type, fn, !!bubble);
            }

            return this;
        },        
        
        /**
         * 判断当前对象是否已经设置 cls 样式
         */
        hasClass: function(cls) {
            // 查找当前样式中，是否存在 cls
            return (' ' + this.className + ' ').indexOf(' ' + cls + ' ') > -1;
        },
        
        /**
         * 切换样式，如果存在就删除，不存在就添加
         */
        toggleClass: function(cls) {
            this.hasClass(cls) ? this.removeClass(cls) : this.addClass(cls);
            return this;
        },

        // 添加样式
        addClass: function(cls) {
            // 添加样子，如果不存在就添加。并删掉多余的空格
            this.hasClass(cls) || (this.className = (this.className + ' ' + cls).trim());
            return this;
        },
        
        // 删除样式
        removeClass: function(cls) {
            // 用替换为空的方式 删除样式
            this.className = (' ' + this.className + ' ').replace(' ' + cls + ' ', ' ').trim();
            return this;
        },
        
        // 清空样式
        emptyClass: function() {
            this.className = '';
            return this;
        },

        // 设置样式
        setClass: function(cls) {
            this.className = cls;
            return this;
        },

        // 设置样式 格式是 JSON
        css: function(css, value) {
            if (typeof css === 'string') {
                // 读取样式
                if (value === undefined) {
                    return doc.defaultView.getComputedStyle(this, null)[css];
                } else {
                    css = that.extend(css, value);
                }
            }

            /**
             * 批量设置CSS属性
             * 如果属性值是数字型，并且检查是否需要带单位的，如果需要都自动加上px
             */
            for (var k in css) {
                this.style[k] = (typeof css[k] === 'number' && !/zIndex|fontWeight|opacity|zoom/i.test(k)) ? css[k] + 'px' : css[k];
            }
            
            return this;
        },

        /**
         * 属性操作
         * 设置的时候 value 是 JSON 对象
         */
        attr: function(attr, value) {
            // 字符
            if (typeof attr === "string") {
                // 读取属性
                if (value === undefined) {
                    return this.getAttribute(attr);
                } else {
                    // 将属性 JSON 对象进行合并
                    attr = that.extend(attr, value);
                }
            }
            
            // 循环设置属性
            for (var k in attr) {
                this.setAttribute(k, attr[k]);
            }
            // 属性文本
            attr.text === undefined || (this.text = attr.text);
            attr.v === undefined || (this.v = attr.v);

            return this;
        },

        // 设置或读取HTML内容
        html: function(value) {
            if (value !== undefined) {
                this.empty();
                // 字符或数字都直接写上去
                if (typeof value == 'string' || typeof value == 'number') {
                    this.innerHTML = value;
                } else {
                    this.appendChild(value);
                }

                return this;
            } else {
                return this.innerHTML;
            }
        },

        // 删除节点
        remove: function() {
            if (this.parentNode) this.parentNode.removeChild(this);
            return this;
        },

        // 将节点设置为空
        empty: function() {
            while(this.firstChild) {
                this.removeChild(this.firstChild);
            }

            return this;
        },
        
        // 是否包含某个子元素
        contains: function(child) {
            /**
             * compareDocumentPosition DOM 方法，返回元素的位置关系
             *
             * 000000         0              元素一致
             * 000001         1              节点在不同的文档（或者一个在文档之外）
             * 000010         2              节点 B 在节点 A 之前
             * 000100         4              节点 A 在节点 B 之前
             * 001000         8              节点 B 包含节点 A
             * 010000         16             节点 A 包含节点 B
             * 100000         32             浏览器的私有使用
             */
            if (this.compareDocumentPosition) {
                // 元素是否相同 || 当前元素是否包含child
                return this == child || !!(this.compareDocumentPosition(child) && 16);
            }
        },

        // 向上（父）节点获取满足条件的元素 tag .class #id
        $p: function(s, ctx) {
            var node = this,
                q = typeof s == 'string' ? rQuery.exec(s) : s;
            ctx = that.$(ctx || 'body');

            while (node && node.nodeName && node != ctx) {
                if (q.nodeName) { if (q == node) return node; }
                else if (q[1] == '#') { if (q[2] == node.id) return node; }
                else if (q[1] == '.') { if (node.hasClass(q[2])) return node; }
                else if (!q[1] && q[2] && (node.nodeName == q[2].toUpperCase())) return node;

                node = that.$(node.parentNode);
            }

            return null;
        },
        
        // 向下级继续查找元素
        $: function (q) {
            return that.$(q, this);
        },

        // 向下继承筛选多个
        $s: function (q) {
            return that.$s(q, this);
        },

        // 插入子节点
        insertChild: function(child, ref) {
            this.insertBefore(child, ref || null);
            return this;
        },
        
        /**
         * 将节点插入到指定位置
         * parent 父节点名称
         */
        insertBeforeChild: function(parent, ref) {
            $(parent).insertBefore(this, ref || null);
            return this;
        },

        // 载入是否完成
        ready: function(fn, ctx) {
            ctx = ctx || this;

            this.bind('load', function() {
                // arguments.callee 调用自身
                this.unbind('load', arguments.callee); 
                fn.call(ctx, this);
            });

            return this;
        }
    };

    // 数组
    this.arrays = {
        // 在数组里查找
        indexOf: function(item, from) {
            var len = this.length, 
                i = (from < 0) ? Math.max(0, len + from) : from || 0;
            // 循环中查找
            for (; i < len; i++) {
                if (this[i] === item) { return i; }
            }

            return -1;
        },
        
        // 循环遍历
        forEach: function(fn, bind) {
            var i = 0, 
                len = this.length;
            // 循环
            for (; i < len; i++) {
                fn.call(bind, this[i], i, this);
            }

            return this;
        },
        
        // 添加事件
        bind: function(type, fn) {
            this.forEach(function(el) {
                el.bind(type, fn);
            });
            return this;
        },

        // 删除事件
        unbind: function(type, fn) {
            this.forEach(function(el) {
                el.unbind(type, fn);
            });
            return this;
        }
    };

    // 字符串
    this.strings = {
        // 删除左右空格
        trim: function() {
            return this.replace(/^\s+|\s|$/g, '');
        },

        // exec 执行脚本 ajax 调用数据也支持 js 脚本
        exec: function(fn, context) {
            var script = '',
                text = this.replace(/<script[^>]*>([\s\S]*?)<\/script>/gi, function(all, code) {
                    script = script + code + "\n";
                    return '';
                });

            if (fn) {
                // 调用函数
                fn.call(context, text);
                // 载入JS
                if (script) $('<script>').attr({'type': 'text/javascript', 'text': script}).insertBeforeChild('body');
            }

            return {'text': text, 'script': script};
        }
    };       

    /**
     * 合并对象
     */
    this.extend = MACO.extend = function(dst, src) {
        // 如果 dst 是字符串
        if (typeof dst === "string") {
            // dst函数名 src 函数体
            if (typeof src === "function") {
                that.elexts[dst] = src;
                // element 对象
                if (window.Element) Element.prototype[dst] = src;
            } else {
                var json = new Object;
                json[dst] = src;
                return json;
            }
        } else {
            // JSON 对象定义的方法
            for (var key in src) {
                if (dst[key] === undefined && src.hasOwnProperty(key)) dst[key] = src[key];
            }
        }

        return dst;
    };

    // 选择元素
    this.query = function(selector, ctx) {
        var els;
        ctx = ctx || doc;
        
        if (selector == null) {
            return [];
        } else if (typeof selector != 'string') { // 本身就是对象的
            if (selector.nodeName == undefined) {
                return selector;
            } else if (selector.$) {
                return [selector];
            } else {
                els = [selector];
            }
        } else if (selector == 'body') { // 获取body
            els = [doc.body];
        } else if (selector == "*") {
            return ctx.getElementsByTagName("*");
        } else if (selector.charAt(0) == "<") {
            // html 类型 <div>
            if (ret = /^<(\w+)\s*\/?>(?:<\/\1>)?$/.exec(selector)) {
                els = [document.createElement(ret[1])];
            } else {
                div.innerHTML = selector; 
                els = div.childNodes;
            }
        } else if (ctx.querySelectorAll) {
            els = ctx.querySelectorAll(selector);
        } else {
            // $("#id li") 类型
            var s = selector.split(" "), ctx = ctx;
            for (var i = 0, len = s.length; i < len; i++) {
                if (s[i]) {
                    els = []; // 初始化为空对象
                    // 用正则匹配 选择
                    els = (that.regx(s[i], ctx));
                    ctx = els[0]; // 将上下文的对象进行替换
                }
            }
        }
            
        return els;
    };

    // 选择器
    that.$ = MACO.$ = function(selector, ctx) {
        var el = that.query(selector, ctx)[0];
       
        if (el) {
            el.$ || that.extend(el, that.elements);
        }
        
        return el == undefined ? [] : el;
    };
    
    // 多选
    that.$s = MACO.$s = function(selector, ctx) {
        var els = that.query(selector, ctx), ret = [], len = els.length;

        for (var i = 0; i < len; i++) {
            if (els[i].nodeType == 1 || els[i].nodeType == 9) {
                els[i].$ || that.extend(els[i], that.elements);
                ret.push(that.extend(els[i], that.elexts));
            }
        }

        return ret;
    };
    
    // 构造url查询,支持url模板与排序
    this.urlQrs = function(data, url) {
        // 定义变量
        var qrs = [], qrs1={}, pos, split = '?', nill = (url == undefined); 
        url = url || '';
        
        // 数据是字符串
        if (typeof data == 'string') {
            // 将数据用&字符号加到URL里面
            url = (url ? url.replace(/[\?#&]+$/g, '') + '&' + data : data);
            data = {}
        } else {
            // {} 占位符方式
            data = that.extend({}, data);
            url = this.urlTpl(data, url);
        }

        // 计算偏移 URL 里面是否有#号
        if ((pos = url.indexOf('#')) > -1) { 
            split = '#';
        } else if ((pos = url.indexOf('?')) < 0) {
            pos = url.indexOf('&');
        }
        
        // 需要偏移
        if (pos > -1) {
            // 将url去掉最后一个符号 然后按 & 切隔
            var urs = url.substr(pos + 1).split('&'), key, value; 
            url = url.substr(0, pos);

            for (var i = urs.length; i--;) {
                if ((pos = urs[i].indexOf('=')) > -1) {
                    key = urs[i].substr(0, pos);
                    value = decodeURIComponent(urs[i].substr(pos + 1));
                } else {
                    key = urs[i];
                    value = '';
                }

                if (data[key] != undefined) {
                    value = data[key];
                    data[key] = undefined;
                }
                if (value && qrs1[key] === undefined) {
                    qrs.push(key + '=' + encodeURIComponent(value)), qrs1[key] = value;
                }
            }
        }

        for (var key in data) {
            if ((nill ? data[key] != undefined : data[key])) {
                qrs.push(key + '=' + encodeURIComponent(data[key]));
            }
        }
        qrs.sort();
        return (url + split + qrs.join('&')).replace(/^[\?#]+|[\?#]+$/g, '');
    };

    // 解析 URL 占位符模板 占位符是 {}
    this.urlTpl = MACO.urlTpl = function(data, url) {
        // 匹配URL里面的 {} 字符
        var tpls = url.match(/{\w+}/g);
        if (tpls) {
            for (var tpl, i = tpls.length; i--;) {
                // 将符号 [{}] 去掉
                tpl = tpls[i].replace(/[{}]/g, '');
                if (data[tpl]) {
                    url = url.replace(tpls[i], data[tpl]);
                    data[tpl] = undefined;
                }
            }
        }

        return url;
    };

    /**
     * ajax 调内的全方法，支持 GET/POST/JSONP
     * method 方法：GET/POST/JSONP
     * fmt 返回的数据格式 xml|json|function
     */
    this.xhr = function(method, url, data, fmt, fn, context) {
        // 如果有数据，并且数据不是对象
        if (data && typeof data != 'object') {
            context = fn; // 将fn重新设置
            fn = fmt; // 匿名方法
            fmt = data;
            data = {};
        }
        
        // 匿名方法
        if (fmt && typeof fmt != 'string') {
            context = fn; // 将fn重新设置
            fn = fmt;
            fmt = null;
        }
        
        // 方法转大写
        method = method.toUpperCase();
        data = data || {};
        
        // 跨域请求 javascript cross domain request
        if (method == 'JSR') {
            var pn = that.xhr.jsr || 'xhr', vn = fmt || that.uniqid(pn); 
            data[pn] = vn;
            // 脚本载入完成之后，触发事件
            $('<script>').ready(function() {
                // jsonp 返回的内容，必须是字符串
                if (fn) {
                    if (window[vn] !== undefined) {
                        typeof(window[vn]) === 'string' ? window[vn].exec(fn, context) : fn.call(context, window[vn]);
                        window[vn] = undefined;
                    } else {
                        // 相当于 将 fn函数 里面的属性用 context 对象替代调用
                        fn.call(context);
                    }
                } 
                // 执行完成之后，删除掉当前载入的JS
                this.remove();
            }).attr({'type': 'text/javascript', 'src':  url}).insertBeforeChild('head');

            return that;
        }

        var req, cmp = false;         
        // 创建 http 对象
        try {
            req = new window.XMLHttpRequest();
        } catch(e) {
            return false;
        }

        try {
            // header 文档类型
            var fmts = {'*': 'text/html, */*', 'xml': 'application/xml', 'json': 'application/json'};
            // 返回的数据格式
            fmt = (fmt || that.xhr.fmt || '*').toLowerCase();
            
            if (method == 'POST' || method == 'PUT') {
                // POST
                req.open(method, that.urlTpl(data, url), !!fn);
                req.setRequestHeader('method', method + ' ' + url + ' HTTP/1.1');
                req.setRequestHeader('Content-Type', 'application/x-www-form-urlencoded');                
            } else {
                // GET 方法，将 data 设置为空
                req.open(method, that.urlQrs(data, url), !!fn);
                data = '';
            }

            req.setRequestHeader("X-Requested-With", "XMLHttpRequest");
            req.setRequestHeader('Accept', fmts[fmt] || fmt);
            
            if (fn && typeof fn == 'function') {
                // ajax 状态改变
                req.onreadystatechange = function() {
                    // cmp 变量防止 多次重复
                    if (req.readyState == 4 && !cmp) {
                        cmp = true;
                        if (fmt == 'xml' || fmt == 'json') {
                            fmt == 'xml' ? data = req.responseXML : eval('data=' + req.responseText);
                            fn.call(context, data, req);
                        } else {
                            req.responseText.exec(function(data){ fn.call(context, data, req)});
                        }
                    }
                }
                req.send(that.urlQrs(data));
            }
        } catch(e) {
            return false;
        }
        
        // 返回 this 或者文本
        return (fn ? that : req.responseText);
    };

    // 配置xhr参数(fmt, jsr)
    this.xhr.set = function (attr, value) {
        if (typeof (attr) == 'string') {
            if (value === undefined) { 
                return that.xhr[attr] || '';
            } else {
                attr = that.extend(attr, value);
            }
        }
        for (var k in attr) {
            that.xhr[k] = attr[k];
        }
        return that;
    };

    // get ajax
    this.get = MACO.get = function(url, data, fn, context) {
        /**
         * 如果 data 不是 object 
         * 将 fn 设置为 context 
         * 清空 data 数据
         */
        typeof data == 'object' || (context = fn, fn = data, data = '');
        return that.xhr('GET', url, data, 'json', fn, context);
    };

    // ajax jsonp xhrid自定义ID
    this.jsonp = MACO.jsonp = function(url, fn, xhrid) {
        var data = xhrid || that.uniqid('JSONP');
        return that.xhr('JSR', url, data, fn, fn);
    }
    
    // 页面载入完成 delay 为延时执行
    this.ready = MACO.ready = function(fn, delay, context) {
        // 兼容老版本的，第二个参数为context
        if (delay && typeof(delay) != 'number') { context = delay; delay = undefined;}

        if (delay) {
            window.bind('load', function() {
                setTimeout(function() {
                    fn.call(context);
                }, delay);
            });
        } else {
            window.bind('load', fn, context);
        }

        return that;
    };

     // 生成唯一标识
    this.uniqid = MACO.uniqid = function(prefix) {
        var uid = new Date().getTime().toString(16);
        uid += Math.floor((1 + Math.random()) * Math.pow(16, (16 - uid.length))).toString(16).substr(1);
        return (prefix || '') + uid;
    };

    // 是否数组
    this.isArray = MACO.isArray = function(obj) {
        return Object.prototype.toString.apply(obj) === '[object Array]';
    };
    
    // 将 arrays 和 strings 对象注册到函数中
    this.extend(Array.prototype, this.arrays);
    this.extend(String.prototype, this.strings);
   
    // 事件
    if (window.Event) this.extend(Event.prototype, this.events);
    // element
    if (window.Element) this.extend(Element.prototype, this.elements);
    
    window.MACO = window.$ = MACO;
}

window.MACO || new MACO;

})(window);
