/**
 * 封装载入 Loading 动作
 *
 * 调用方法：
 * $.loading().init(); // 初始化
 * $.loading().open(); // 打开Loading状态
 * $.loading().close(); // 关闭
 */

MACO.loading = function() {
    if (MACO.Loading == undefined) MACO.Loading = new Loading();
    return MACO.Loading;
};

function Loading() {
    this.wrap = null;
    this.obj = null;

    this.width = 150;
    this.height = 150;    
}

Loading.prototype = {
    // 初始化load状态
    init: function() {
        // 根据当前的分辨率 设置为居中        
        // 页面宽高
        var iw = window.innerWidth,
            ih = window.innerHeight;

        var left = parseInt((iw - this.width) / 2);
        var top = parseInt((ih - this.height) / 2);
        
        var div = document.createElement("div");
        // 设置样式
        div.css({"position": "absolute", "zIndex": "10001", "visibility": "hidden", "opacity": 0.8, "backgroundColor": "#4a494a", "borderRadius" : "10px", "width": this.width, "height": this.height, "left": left, "top": top});
        // 插入到 BODY
        $("body").insertChild(div);
        
        // 创建loadingDIV
        var loading = document.createElement("div");
        loading.setClass("loading2");
        div.insertChild(loading);

        // 显示进度的loading图标
        var loadingImg = document.createElement("div");
        loadingImg.setClass("loading_img");
        loading.insertChild(loadingImg);

        // 创建一个DIV
        var wrap = document.createElement("div");
        wrap.css({"position": "absolute", "zIndex": "10000", "visibility": "hidden", "opacity": 0.2, "backgroundColor": "#908E8F", "top": 0, "left": 0, "width": iw, "height": ih});
        $("body").insertChild(wrap);

        // 绑定事件
        div.bind("touchstart", function(e) {
            e.preventDefault();
        });
        wrap.bind("touchstart", function(e) {
            e.preventDefault();
        });

        this.obj = div;
        this.wrap = wrap;        
    },

    // 横竖屏的时候 触发
    orientChange: function() {
        // 页宽
        var iw = window.innerWidth;
        var ih = window.innerHeight;

        var left = parseInt((iw - this.obj.offsetWidth) / 2);
        var top = parseInt((ih - this.obj.offsetHeight) / 2);

        this.obj.css({"left": left, "top": top});
        // 设置宽高为当前屏幕
        this.wrap.css({"width": iw, "height": ih});
    },
    
    // 打开loading
    open: function(tipmsg) {
        var tipmsg = tipmsg || "正在处理中...";
        
        // 显示隐藏的 Loading 对象
        this.obj.css({"visibility": "visible"});
        this.wrap.css({"visibility": "visible"});

        var div = document.createElement("div");
        div.setClass("loading_tipmsg").html(tipmsg);
        this.obj.insertChild(div);
    },

    // 关闭loading状态
    close: function() {
        // 删除掉创建的div
        try {
            this.obj.remove();
            this.wrap.remove();
        } catch(e) {}
    }
};

/**
 * 消息提示
 *
 * $.tip().show("消息提示");
 */

MACO.tip = function(conf) {
    if (MACO.Tip == undefined) MACO.Tip = new Tip(conf);
    return MACO.Tip;
};

/**
 * 提示
 * conf 是 {} 对象，可设置宽度高度等
 */
function Tip(conf) {
    conf = conf || {};
    
    this.wrap = null;
    this.obj = null; // 句柄

    // 宽高
    this.width = conf.width || 220;
    this.height = conf.height || 30;

    // 时长
    this.showtime = conf.showtime || 2000; // 默认2秒    
}

Tip.prototype = {
    // 初始化
    _init: function() {
        // 样式
        var objcss = {"position": "absolute", "font-size": "14px", "padding": "5px 10px", "line-height": this.height, "text-align": "center", "zIndex": "99999", "visibility": "hidden", "color": "white", "background-color": "#4a494a", "box-shadow": "1px 1px 5px #999"};

        var div = document.createElement("div");
        div.css(objcss);
        $("body").insertChild(div);
        
        var iw = window.innerWidth,
            ih = window.innerHeight;

        // 创建一个DIV
        var wrap = document.createElement("div");
        wrap.css({"position": "absolute", "zIndex": "7776", "visibility": "hidden", "opacity": 0.2, "backgroundColor": "#C4C4C4", "top": 0, "left": 0, "width": iw, "height": ih});
        $("body").insertChild(wrap);

        // 停止默认事件
        div.bind("touchstart", function(e) {
            e.preventDefault();   
        });
        wrap.bind("touchstart", function(e) {
            e.preventDefault();
        });

        this.obj = div;
        this.wrap = wrap;
    },
    
    // 过程中
    progress: function() {
        this.showtime = 30000;
        this.show("正在处理中...");
    },

    // 主动结速提示
    close: function() {
        var _this = this;
        try { // 删除对象
            _this.obj.remove(); 
            _this.wrap.remove();
            _this.obj = null;
            _this.wrap = null;
        } catch(e) {}
    },

    // 提示信息
    show: function(msg) {
        var _this = this;

        // 初始化
        if (!_this.obj) {
            _this._init();
        }

        // 显示文本
        _this.obj.html(msg);

        var left = parseInt((window.innerWidth - _this.obj.offsetWidth) / 2);
        var top = parseInt((window.innerHeight - _this.obj.offsetHeight) / 2);
        var objcss = {"left": left, "top": top, "visibility": "visible"};
        _this.obj.css(objcss); // 将对象居中，并显示
        _this.wrap.css({"visibility": "visible"});
        
        // 2秒之后，关闭
        setTimeout(function() {
            try { 
                _this.obj.remove(); // 删除对象
                _this.wrap.remove();
                _this.obj = null;
                _this.wrap = null;
            } catch(e) {}
        }, _this.showtime);

        return _this;
    },

    // 进度提示
    showWithInverse: function(msg) {
        var _this = this;

        if (!_this.obj) {
            _this._init();
        }

        _this.showtime = 1500;
        
        _this.wrap.css({"visibility": "visible"});
        _this.obj.css({"backgroundColor": "rgba(0, 0, 0, .75)", "padding": "5px 10px"});
        
        // 显示文本
        _this.obj.html(msg);

        var left = parseInt((window.innerWidth - _this.obj.offsetWidth) / 2);
        var top = parseInt((window.innerHeight - _this.obj.offsetHeight) / 2);
        var objcss = {"left": left, "top": top, "visibility": "visible"};
        _this.obj.css(objcss); // 将对象居中，并显示

        // 1秒之后，关闭
        setTimeout(function() {
            try {
                _this.obj.remove();
                _this.wrap.remove();
                _this.obj = null;
                _this.wrap = null;
            } catch (e) {}
        }, _this.showtime);

        return _this;
    },

    // 显示消息之后的事件
    after: function(recall) {
        if (recall && typeof recall == 'function') {
			recall();
		} else if(recall) {
			eval(recall);
		}
    }
};

// 滑动封装

(function(){
var m = Math,
    // 按位右移
	mround = function (r) { return r >> 0; },
	vendor = (/webkit/i).test(navigator.appVersion) ? 'webkit' :
		(/firefox/i).test(navigator.userAgent) ? 'Moz' :
		'opera' in window ? 'O' : '',

    // Browser capabilities
    isAndroid = (/android/gi).test(navigator.appVersion),
    isIDevice = (/iphone|ipad/gi).test(navigator.appVersion),
    isPlaybook = (/playbook/gi).test(navigator.appVersion),
    isTouchPad = (/hp-tablet/gi).test(navigator.appVersion),

    // 检查是否支持 滑动，3D动画
    has3d = 'WebKitCSSMatrix' in window && 'm11' in new WebKitCSSMatrix(),
    hasTouch = 'ontouchstart' in window && !isTouchPad,
    hasTransform = vendor + 'Transform' in document.documentElement.style,
    hasTransitionEnd = isIDevice || isPlaybook,

    // 下一个动画
	nextFrame = (function() {
	    return window.requestAnimationFrame
			|| window.webkitRequestAnimationFrame
			|| window.mozRequestAnimationFrame
			|| window.oRequestAnimationFrame
			|| window.msRequestAnimationFrame
			|| function(callback) { return setTimeout(callback, 17); }
	})(),
	cancelFrame = (function () {
	    return window.cancelRequestAnimationFrame
            || window.cancelAnimationFrame
			|| window.webkitCancelAnimationFrame
			|| window.webkitCancelRequestAnimationFrame
			|| window.mozCancelRequestAnimationFrame
			|| window.oCancelRequestAnimationFrame
			|| window.msCancelRequestAnimationFrame
			|| clearTimeout
	})(),

	// Events
	RESIZE_EV = 'onorientationchange' in window ? 'orientationchange' : 'resize',
	START_EV = hasTouch ? 'touchstart' : 'mousedown',
	MOVE_EV = hasTouch ? 'touchmove' : 'mousemove',
	END_EV = hasTouch ? 'touchend' : 'mouseup',
	CANCEL_EV = hasTouch ? 'touchcancel' : 'mouseup',

	// Helpers
	trnOpen = 'translate' + (has3d ? '3d(' : '('),
	trnClose = has3d ? ',0)' : ')',

	// Constructor
	mScroll = function (el, options) {
		var that = this,
			doc = document,
			i;

		that.wrapper = typeof el == 'object' ? el : doc.getElementById(el);
		that.wrapper.style.overflow = 'hidden';
		that.scroller = that.wrapper.children[0];

		// Default options
		that.options = {
			hScroll: false, // 是否横向滚动
			vScroll: true, // 是否垂直滚动
			x: 0,
			y: 0,
            topOffset: 0, // 默认上边距，在做下拉刷新时有用
			bounce: true, // 启用或禁用边界的反弹
			bounceLock: false, // //当内容少于滚动是否可以反弹
			momentum: true, // 启用或禁用惯性
			lockDirection: true, // 滑动方向锁定
			useTransform: true,    // 变换效果
			useTransition: false,  // 过渡效果
            handleClick: false, // 绑定 click 事件，在滑动结束的时候 触发
            handleTap: true, // 绑定一个 tap 事件

			// Events
			onRefresh: null,
			onBeforeScrollStart: function (e) { e.preventDefault(); },
			onScrollStart: null,
			onBeforeScrollMove: null,
			onScrollMove: null,
			onBeforeScrollEnd: null,
			onScrollEnd: null,
			onTouchEnd: null,
			onDestroy: null
		};

		// 使用自定义的方法
		for (i in options) that.options[i] = options[i];

		// 可以自定义初始位置
		that.x = that.options.x;
		that.y = that.options.y;

		// 基本选项值
		that.options.useTransform = hasTransform ? that.options.useTransform : false; // 是否使用变换
		that.options.hScrollbar = that.options.hScroll && that.options.hScrollbar; // 隐藏滚动条
		that.options.vScrollbar = that.options.vScroll && that.options.vScrollbar; // 显示滚动条
		that.options.useTransition = hasTransitionEnd && that.options.useTransition; // 动画

		/**
         * 设置一些默认的动画样式
         * transition-property 指定要进行过渡的css属性
         * transition-duration 过渡的持续时间
         * transform-origin 变换的属性
         */
		that.scroller.style[vendor + 'TransitionProperty'] = that.options.useTransform ? '-' + vendor.toLowerCase() + '-transform' : 'top left';
		that.scroller.style[vendor + 'TransitionDuration'] = '0';
		that.scroller.style[vendor + 'TransformOrigin'] = '0 0';

        // 使用过渡效果 就设置 贝塞尔曲线 函数值
		if (that.options.useTransition) that.scroller.style[vendor + 'TransitionTimingFunction'] = 'cubic-bezier(0.33,0.66,0.66,1)';
		
        // 使用变换 就要判断是 2d/3d 变换属性
		if (that.options.useTransform) that.scroller.style[vendor + 'Transform'] = trnOpen + that.x + 'px,' + that.y + 'px' + trnClose;
		else that.scroller.style.cssText += ';position:absolute;top:' + that.y + 'px;left:' + that.x + 'px';

        // 刷新
		that.refresh();
        
        // 帮定窗口缩放事件
		that._bind(RESIZE_EV, window);
		that._bind(START_EV); // 滑动开始

        // 不能滑动，就绑定一个鼠标离开事件
		if (!hasTouch) that._bind('mouseout', that.wrapper); 
	};

// Prototype
mScroll.prototype = {
	enabled: true,
	x: 0,
	y: 0,
	steps: [],
	scale: 1,
	
	handleEvent: function (e) {
		var that = this;
		switch(e.type) {
			case START_EV:
				if (!hasTouch && e.button !== 0) return;
				that._start(e);
				break;
			case MOVE_EV: that._move(e); break;
			case END_EV:
			case CANCEL_EV: that._end(e); break;
			case RESIZE_EV: that._resize(); break;
			case 'mouseout': that._mouseout(e); break;
			case 'webkitTransitionEnd': that._transitionEnd(e); break;
		}
	},
    
    // 重置
	_resize: function () {
		this.refresh();
	},
	
    // 移动到坐标
	_pos: function (x, y) {
        // 如果隐藏滚动条就全置0
		x = this.hScroll ? x : 0;
		y = this.vScroll ? y : 0;

        // 3d 变化
		if (this.options.useTransform) {
			this.scroller.style[vendor + 'Transform'] = trnOpen + x + 'px,' + y + 'px' + trnClose + ' scale(' + this.scale + ')';
		} else {
            // 按位右移
			x = mround(x);
			y = mround(y);

            // 设置新的坐标
			this.scroller.style.left = x + 'px';
			this.scroller.style.top = y + 'px';
		}

		this.x = x;
		this.y = y;
	},
    
    // 滑动开始
	_start: function (e) {
		var that = this,
			point = hasTouch ? e.touches[0] : e,
			matrix, x, y;

		if (!that.enabled) return;
        
		if (that.options.onBeforeScrollStart) that.options.onBeforeScrollStart.call(that, e);
		
        // 使用贝塞尔插值变换 就将过滤时间设置为0
		if (that.options.useTransition) that._transitionTime(0);

		that.moved = false;
		that.animating = false;
		that.zoomed = false;
		that.distX = 0;
		that.distY = 0;
		that.absDistX = 0;
		that.absDistY = 0;
		that.dirX = 0;
		that.dirY = 0;
        
        // 是否启用惯性
		if (that.options.momentum) {
            // 使用变换属性
			if (that.options.useTransform) {
				// 山寨方式实现 CSSMatrix 的矩阵变换 基于 x,y 坐标的变换
				matrix = getComputedStyle(that.scroller, null)[vendor + 'Transform'].replace(/[^0-9-.,]/g, '').split(',');
                x = matrix[4] * 1;
				y = matrix[5] * 1;
			} else {
                // 使用 left/top 属性
				x = getComputedStyle(that.scroller, null).left.replace(/[^0-9-]/g, '') * 1;
				y = getComputedStyle(that.scroller, null).top.replace(/[^0-9-]/g, '') * 1;
			}
			
            // 只要开始动了，就停掉之前的动作
			if (x != that.x || y != that.y) {
				if (that.options.useTransition) that._unbind('webkitTransitionEnd');
				else cancelFrame(that.aniTime);
                // 将动画设置为空
				that.steps = [];
                // 移动到指定坐标
				that._pos(x, y);
			}
		}
        
        // 初始化x,y
		that.startX = that.x;
		that.startY = that.y;
		that.pointX = point.pageX;
		that.pointY = point.pageY;

		that.startTime = e.timeStamp || Date.now();

		if (that.options.onScrollStart) that.options.onScrollStart.call(that, e);

		that._bind(MOVE_EV);
		that._bind(END_EV);
		that._bind(CANCEL_EV);
	},
	
    // 滑动过程中
	_move: function (e) {
		var that = this,
			point = hasTouch ? e.touches[0] : e,
			deltaX = point.pageX - that.pointX,
			deltaY = point.pageY - that.pointY,
			newX = that.x + deltaX,
			newY = that.y + deltaY,
			timestamp = e.timeStamp || Date.now();

		if (that.options.onBeforeScrollMove) that.options.onBeforeScrollMove.call(that, e);

		that.pointX = point.pageX;
		that.pointY = point.pageY;
        
        /**
         * 边界反弹效果
         * >0 的表示拉过头了 否则就是 max
         */
		if (newX > 0 || newX < that.maxScrollX) {
			newX = that.options.bounce ? that.x + (deltaX / 2) : newX >= 0 || that.maxScrollX >= 0 ? 0 : that.maxScrollX;
		}
		if (newY > that.minScrollY || newY < that.maxScrollY) { 
			newY = that.options.bounce ? that.y + (deltaY / 2) : newY >= that.minScrollY || that.maxScrollY >= 0 ? that.minScrollY : that.maxScrollY;
		}

		that.distX += deltaX;
		that.distY += deltaY;
		that.absDistX = m.abs(that.distX);
		that.absDistY = m.abs(that.distY);

		if (that.absDistX < 6 && that.absDistY < 6) {
			return;
		}

		// 锁定滑动方向
		if (that.options.lockDirection) {
			if (that.absDistX > that.absDistY + 5) {
				newY = that.y;
				deltaY = 0;
			} else if (that.absDistY > that.absDistX + 5) {
				newX = that.x;
				deltaX = 0;
			}
		}

		that.moved = true;
		that._pos(newX, newY);
		that.dirX = deltaX > 0 ? -1 : deltaX < 0 ? 1 : 0;
		that.dirY = deltaY > 0 ? -1 : deltaY < 0 ? 1 : 0;
        
        // 持续滑动超过 300ms 就重置起始坐标，动作就是手指快速的在屏幕上滑动很长的位置
		if (timestamp - that.startTime > 300) {
			that.startTime = timestamp;
			that.startX = that.x;
			that.startY = that.y;
		}
		
		if (that.options.onScrollMove) that.options.onScrollMove.call(that, e);
	},
	
    // 滑动结束
	_end: function (e) {
		if (hasTouch && e.touches.length != 0) return;

		var that = this,
			point = hasTouch ? e.changedTouches[0] : e,
			target, ev,
			momentumX = { dist:0, time:0 },
			momentumY = { dist:0, time:0 },
			duration = (e.timeStamp || Date.now()) - that.startTime,
			newPosX = that.x,
			newPosY = that.y,
			newDuration;
        
        // 速度之后，解除绑定事件
		that._unbind(MOVE_EV);
		that._unbind(END_EV);
		that._unbind(CANCEL_EV);

		if (that.options.onBeforeScrollEnd) that.options.onBeforeScrollEnd.call(that, e);

		if (!that.moved) {
			if (hasTouch) {
                // 启用click事件句柄才绑定
                if (that.options.handleClick) {
                    that.doubleClickTimer = setTimeout(function () {
						that.doubleClickTimer = null;

                        // 滑动停止之后，点击屏幕 用JS模仿鼠标事件
                        target = point.target;
                        while (target.nodeType != 1) target = target.parentNode;

                        if (target.tagName != 'SELECT' && target.tagName != 'INPUT' && target.tagName != 'TEXTAREA') {
                            ev = document.createEvent('MouseEvents');
                            ev.initMouseEvent('click', true, true, e.view, 1,
                                point.screenX, point.screenY, point.clientX, point.clientY,
                                e.ctrlKey, e.altKey, e.shiftKey, e.metaKey,
                                0, null);
                            ev._fake = true;
                            target.dispatchEvent(ev);
                        }
                    }, 0);
                }
                
                // 绑定 click 就绑定一个tap事件
                if (that.options.handleTap) {
                    // tap
                    that.doubleTapTimer = setTimeout(function () {
						that.doubleTapTimer = null;

                        target = point.target;
                        while (target.nodeType != 1) target = target.parentNode;

                        var ev = document.createEvent('Event');
                        ev.initEvent('tap', true, true);
                        target.dispatchEvent(ev);

                    }, 0);
                }
			}

			that._resetPos(200);

			if (that.options.onTouchEnd) that.options.onTouchEnd.call(that, e);
			return;
		}
        
        // 滑动开始时间与结束时间，小于300ms 并且启用了惯性
		if (duration < 300 && that.options.momentum) {
			momentumX = newPosX ? that._momentum(newPosX - that.startX, duration, -that.x, that.scrollerW - that.wrapperW + that.x, that.options.bounce ? that.wrapperW : 0) : momentumX;
			momentumY = newPosY ? that._momentum(newPosY - that.startY, duration, -that.y, (that.maxScrollY < 0 ? that.scrollerH - that.wrapperH + that.y - that.minScrollY : 0), that.options.bounce ? that.wrapperH : 0) : momentumY;

			newPosX = that.x + momentumX.dist;
			newPosY = that.y + momentumY.dist;

 			if ((that.x > 0 && newPosX > 0) || (that.x < that.maxScrollX && newPosX < that.maxScrollX)) momentumX = { dist:0, time:0 };
 			if ((that.y > that.minScrollY && newPosY > that.minScrollY) || (that.y < that.maxScrollY && newPosY < that.maxScrollY)) momentumY = { dist:0, time:0 };
		}

		if (momentumX.dist || momentumY.dist) {
			newDuration = m.max(m.max(momentumX.time, momentumY.time), 10);
            // 移动到指定位置
			that.scrollTo(mround(newPosX), mround(newPosY), newDuration);

			if (that.options.onTouchEnd) that.options.onTouchEnd.call(that, e);
			return;
		}

		that._resetPos(200);
		if (that.options.onTouchEnd) that.options.onTouchEnd.call(that, e);
	},
	
    // 重置位移
	_resetPos: function (time) {
		var that = this,
			resetX = that.x >= 0 ? 0 : that.x < that.maxScrollX ? that.maxScrollX : that.x,
			resetY = that.y >= that.minScrollY || that.maxScrollY > 0 ? that.minScrollY : that.y < that.maxScrollY ? that.maxScrollY : that.y;
        
		if (resetX == that.x && resetY == that.y) {
			if (that.moved) {
				if (that.options.onScrollEnd) that.options.onScrollEnd.call(that);		// 执行自定义滑动结束回调事件
				that.moved = false;
			}

			return;
		}
        
        // 滑动到 x,x
		that.scrollTo(resetX, resetY, time || 0);
	},
	
    // 桌面开发用的，鼠标离开的时候 要停止事件
	_mouseout: function (e) {
		var t = e.relatedTarget;

		if (!t) {
			this._end(e);
			return;
		}

		while (t = t.parentNode) if (t == this.wrapper) return;
		
		this._end(e);
	},
    
    // 元素变换完成之后，触发的事件
	_transitionEnd: function (e) {
		var that = this;

		if (e.target != that.scroller) return;

		that._unbind('webkitTransitionEnd');
		
		that._startAni();
	},

	/**
	 * 开始动画
	 */
	_startAni: function () {
		var that = this,
			startX = that.x, startY = that.y,
			startTime = Date.now(),
			step, easeOut,
			animate;
        
        // 正在执行中
		if (that.animating) return;
        
        // 如果步骤没有了，就重置位置 400ms 延时
		if (!that.steps.length) {
			that._resetPos(400);
			return;
		}
        
        // 总是取第一步
		step = that.steps.shift();
        
        // 要移动的 x,y 就是当前位置，那就把时间设置为0
		if (step.x == startX && step.y == startY) step.time = 0;
        
        // 设置状态
		that.animating = true;
		that.moved = true;
        
        // 如果启用 贝塞尔插值变换
		if (that.options.useTransition) {
			that._transitionTime(step.time);
			that._pos(step.x, step.y);
			that.animating = false;
			if (step.time) that._bind('webkitTransitionEnd');
			else that._resetPos(0);
			return;
		}
		
        // 自定义动画
		animate = function () {
			var now = Date.now(),
				newX, newY;
            
            // 时间已经过了
			if (now >= startTime + step.time) {
				that._pos(step.x, step.y);
				that.animating = false;
				if (that.options.onAnimationEnd) that.options.onAnimationEnd.call(that);			// 触发动画完成之后的回调事件
                // 继续回调
				that._startAni();
				return;
			}
            
            // 时间差/延时-1
			now = (now - startTime) / step.time - 1;
            // 逐渐减速，用开方实现
			easeOut = m.sqrt(1 - now * now);
            // 要移动的坐标 - 当前坐标 * 延时 + 当前坐标
			newX = (step.x - startX) * easeOut + startX;
			newY = (step.y - startY) * easeOut + startY;
            // 位移
			that._pos(newX, newY);
            // 如果还在执行中，用浏览器内部动画方法
			if (that.animating) that.aniTime = nextFrame(animate);
		};
		// 触发自定义动画
		animate();
	},
    
    // 过渡的持续时间
	_transitionTime: function (time) {
		this.scroller.style[vendor + 'TransitionDuration'] = time + 'ms';
	},
	
    // 计算出惯性值
	_momentum: function (dist, time, maxDistUpper, maxDistLower, size) {
		var deceleration = 0.0006,
			speed = m.abs(dist) / time,
			newDist = (speed * speed) / (2 * deceleration),
			newTime = 0, outsideDist = 0;

		// Proportinally reduce speed if we are outside of the boundaries 
		if (dist > 0 && newDist > maxDistUpper) {
			outsideDist = size / (6 / (newDist / speed * deceleration));
			maxDistUpper = maxDistUpper + outsideDist;
			speed = speed * maxDistUpper / newDist;
			newDist = maxDistUpper;
		} else if (dist < 0 && newDist > maxDistLower) {
			outsideDist = size / (6 / (newDist / speed * deceleration));
			maxDistLower = maxDistLower + outsideDist;
			speed = speed * maxDistLower / newDist;
			newDist = maxDistLower;
		}

		newDist = newDist * (dist < 0 ? -1 : 1);
		newTime = speed / deceleration;

		return { dist: newDist, time: mround(newTime) };
	},
    
    // 元素的位置
	_offset: function (el) {
		var left = -el.offsetLeft,
			top = -el.offsetTop;
			
		while (el = el.offsetParent) {
			left -= el.offsetLeft;
			top -= el.offsetTop;
		} 

		return { left: left, top: top };
	},

    // 绑定事件
	_bind: function (type, el, bubble) {
		(el || this.scroller).addEventListener(type, this, !!bubble);
	},
    
    // 解除绑定
	_unbind: function (type, el, bubble) {
		(el || this.scroller).removeEventListener(type, this, !!bubble);
	},


	// 销毁对象，释放内存
	destroy: function () {
		var that = this;

		that.scroller.style[vendor + 'Transform'] = '';

		// Remove the event listeners
		that._unbind(RESIZE_EV, window);
		that._unbind(START_EV);
		that._unbind(MOVE_EV);
		that._unbind(END_EV);
		that._unbind(CANCEL_EV);
		that._unbind('mouseout', that.wrapper);
		if (that.options.useTransition) that._unbind('webkitTransitionEnd');
		
		if (that.options.onDestroy) that.options.onDestroy.call(that);
	},

    // 刷新
	refresh: function () {
		var that = this,
			offset;
        
        // wrapper 当前宽高
		that.wrapperW = that.wrapper.clientWidth;
		that.wrapperH = that.wrapper.clientHeight;

        // 最小可滑动的Y轴 在做上拉刷新有效
        that.minScrollY = -that.options.topOffset || 0;

        // 要滑动元素宽高
		that.scrollerW = that.scroller.offsetWidth;
		that.scrollerH = that.scroller.offsetHeight + that.minScrollY;
        
        // 最大可以滑动的坐标值
		that.maxScrollX = that.wrapperW - that.scrollerW;
		that.maxScrollY = that.wrapperH - that.scrollerH + that.minScrollY;

		that.dirX = 0;
		that.dirY = 0;

        if (that.options.onRefresh) that.options.onRefresh.call(that);
        
        // 隐藏滚动条
		that.hScroll = that.options.hScroll && that.maxScrollX < 0;
        /**
         * 显示滚动条
         * 没有锁定 & 没有设置隐藏 || 可以滑动的高大于wapper的高
         */
		that.vScroll = that.options.vScroll && (!that.options.bounceLock && !that.hScroll || that.scrollerH > that.wrapperH);
        
        // 当前位置
		offset = that._offset(that.wrapper);

        // 左边距和上边距
		that.wrapperOffsetLeft = -offset.left;
		that.wrapperOffsetTop = -offset.top;
        
        // 滑动延时设置为 0
		that.scroller.style[vendor + 'TransitionDuration'] = '0';
        
        // 重置
		that._resetPos(200);
	},
    
    // 滑动到指定位置
	scrollTo: function (x, y, time, relative) {
		var that = this,
			step = x,
			i, l;
        
        // 先停止默认动作
		that.stop();
        
        // 如果 x 不是对象，就重置成对象
		if (!step.length) step = [{ x: x, y: y, time: time, relative: relative }];
		
		for (i=0, l=step.length; i<l; i++) {
            // 相对位置
			if (step[i].relative) { 
                // 坐标等于当前的 x,y 减去要移动到的
                step[i].x = that.x - step[i].x; 
                step[i].y = that.y - step[i].y; 
            }
            // 写入到动画数组列队
			that.steps.push({ x: step[i].x, y: step[i].y, time: step[i].time || 0 });
		}
        
        // 执行动画
		that._startAni();
	},

    // 滑动到指定元素
	scrollToElement: function (el, time) {
		var that = this, pos;
		el = el.nodeType ? el : that.scroller.querySelector(el);
		if (!el) return;
        
		pos = that._offset(el);
		pos.left += that.wrapperOffsetLeft;
		pos.top += that.wrapperOffsetTop;

		pos.left = pos.left > 0 ? 0 : pos.left < that.maxScrollX ? that.maxScrollX : pos.left;
		pos.top = pos.top > that.minScrollY ? that.minScrollY : pos.top < that.maxScrollY ? that.maxScrollY : pos.top;
		time = time === undefined ? m.max(m.abs(pos.left)*2, m.abs(pos.top)*2) : time;

		that.scrollTo(pos.left, pos.top, time);
	},
    
    // 禁止滑动
	disable: function () {
		this.stop();
		this._resetPos(0);
		this.enabled = false;

		// 禁止滑动之事，注销事件
		this._unbind(MOVE_EV);
		this._unbind(END_EV);
		this._unbind(CANCEL_EV);
	},
	
    // 启用
	enable: function () {
		this.enabled = true;
	},
	
    // 停止动作
	stop: function () {
		cancelFrame(this.aniTime);
		this.steps = [];
		this.moved = false;
		this.animating = false;
	}
};

window.mScroll = mScroll;

})();

