/**
 * 章节列表对象的封装
 * 文章阅读页面点击的时候，显示一个章节列表
 */

(function() {

    var bid = 0; // 书号
    var aindex = 0; // 章节索引
    var ctotal = 0; // 总共章节数

    var page = 1; // 当前页号
    var page_size = 100; // 每页100条
    var page_total = 1; // 分页数量

    var chapters = {}; // 当前章节列表
        
    var myScroll;

    function myScrollLoaded() {
        myScroll = new mScroll("dircontent", {
            hScroll: false,
            onBeforeScrollStart: function(e) {
                var target = e.target;
                // 行焦点
                var data_focus = target.attr("data-focus");
                if (data_focus) $("#"+data_focus).focus();

                if (target.tagName != 'INPUT') {
                    e.preventDefault();
                }

                // 隐藏下拉菜单
                if (target.tagName != "P") {
                    hideDropDown("pageno");
                }
            }
        });
    }

    $(document).bind("touchmove", function (e) { e.preventDefault(); }, false);
    $(document).bind("DOMContentLoaded", function() {        
        myScrollLoaded();

        // 初始化 bid 和 aindex
        bid = parseInt(dataStorage.getItem("read.bid"));
        aindex = parseInt(nlg(dataStorage.getItem("read.aindex"), 1));
        if (aindex <= 0) aindex = 1;

        // 当前页号
        page = Math.ceil(aindex / page_size);
        if (page < 1) page = 1;
        
        try {
            // 读取章节
            var bookchapter = dataStorage.getItem("book.chapter."+bid);
            if (bookchapter) {
                chapters = JSON.parse(bookchapter);
                ctotal = chapters.length;
            
                // 为了防止章节过多，无法显示全部的章节，最多显示8页
                if (chapters.length && chapters.length > 800) {
                    page_size = Math.ceil(chapters.length / 8);
                }
                // 释放变量
                bookchapter = null;
            }
        } catch(e) {}
        
        // 根据章节数量，生成分页选项
        if (chapters.length > page_size) {
            page_total = Math.ceil(chapters.length / page_size);
            var pageno_more = $("#pageno_more"), p, start = 1, end = page_size;
            pageno_more.empty();
            for (var i = 1; i <= page_total; i++) {
                if (i > 1) {
                    start = end + 1;
                    end = i * page_size;
                    
                    // 最后一部分
                    if (i == page_total) {
                        end = chapters.length;
                    }
                }

                p = document.createElement("p");
                p.attr({"data-value": i}).html(start+'-'+end+'章');
                pageno_more.insertChild(p);
            }
            showDropDown("pageno", page, function(go_page) {
                if (go_page > 0) {
                    // 取消之前被选中的样式
                    $("#tab_bookmark").removeClass("current");
                    // 设置选中样式
                    $("#tab_readdir").addClass("current");
                    page = go_page;
                    fillContent(go_page);
                }
            });
        }

        showReadDir();        
    }, false);
    
    $("#dirscroll").bind(CLICK_EVENT, function(e) {
        e.preventDefault();
        onTouchEvent(e);
    });

    $("#dirtitle").bind(START_EVENT, function(e) {
        e.preventDefault();
        var id = e.target.attr("data-id");
        if (id == "back") {
            var backpath = backPagePath("get");
            if (backpath) {
                ref = backpath;
                backPagePath("remove");
            } else {
                ref = "read.html";
            }
            go(ref);
        } else if (id == "backtop") { // 点中间回顶部
            if (Math.abs(myScroll.y) > 100) {
                myScroll.scrollTo(0, 52, 500);
            }
        }
    });

    // 切换
    $("#tabpanel").bind(START_EVENT, function(e) {
        e.preventDefault();
        var target = e.target;
        var id = target.attr("data-id");

        // 取消之前被选中的样式
        $("#tabpanel .current").removeClass("current");
        // 设置选中样式
        target.addClass("current");

        if (id == "bookmark") {
            showBookMark();
        } else {
            showReadDir();
        }
    });

    // 分页
    $("#bottom-panel").bind(START_EVENT, function(e) {
        var target = e.target;
        var data_id = target.attr("id");

        var defaultSelected = function() {
            // 设置默认选中的
            var defaultObj = $("#pageno_more").$s("p");
            if (defaultObj && defaultObj.length > page - 1) {
                var target = defaultObj[page-1];
                if (target) {
                    var data_value = target.attr("data-value"),
                        html = target.html();
                    $("#pageno").attr({"data-value": data_value}).html(html);
                }
            }
        };

        if (data_id == "dirpage-prev") { // 上一页
            if (page > 1) {
                page--;
                fillContent(page);
                defaultSelected();                
            } else {
                showMsg("已经是第一页了");
            }
        } else if (data_id == "dirpage-next") { // 下一页
            if (page < page_total) {
                page++;
                fillContent(page);
                defaultSelected();
            } else {
                showMsg("已经是最后一页了");
            }
        }
    });

    // 显示书签
    function showBookMark() {
        var dirlist = $("#dirlist"), doc = document, li;
        dirlist.empty();
        dirlist.addClass("bookmark_list");

        myScroll.scrollTo(0, 0, 0);

        var bookmark_id = "book.mark.record."+bid;
        var bookmark = dataStorage.getItem(bookmark_id);
        if (bookmark) {
            bookmark = JSON.parse(bookmark);
            
            var html = '', firstObj, i = 1;
            
            for (var data in bookmark) {
                html += '<div>'+ i +'.'+ bookmark[data].title+'</div>';
                html += '<div class="clearfix"><div class="page">第'+bookmark[data].page+'页</div><div class="time">'+bookmark[data].time+'</div></div>';
                
                li = doc.createElement("li");
                li.addClass("item").attr({"id": "aindex_"+bookmark[data].aindex, "data-aindex": bookmark[data].aindex, "data-vip": 0, "data-page": bookmark[data].page});
                li.html(html);
                dirlist.insertBefore(li, dirlist.firstChild);

                html = '';
                i++;
            }
        } else {
            li = doc.createElement("li");
            li.html("还没有为本作品添加过书签，在阅读章节时，可随时添加书签位置方便下次继续阅读！");
            dirlist.appendChild(li);
        }

        myScroll.refresh();
    }

    // 显示章节目录
    function showReadDir() {
        fillContent(page);
        $("#dirtotal_article").html('共'+ctotal+'章');
    }

    // 将章节内容 渲染到列表
    function fillContent(page) {
        var dirlist = $("#dirlist"), 
            li, _aid, _aindex, _vip, _artstat, _buystat;
        
        if (dirlist.html() != "") {
            dirlist.empty();
        }
        dirlist.removeClass("bookmark_list");

        // 计算页码
        var start = 0, end = page * page_size;
        if (page > 1) {
            start = (page - 1) * page_size;
        }
        if (end > chapters.length) {
            end = chapters.length;
        }
        
        // 章节列表是否加载成功
        if (chapters && chapters.length > 0) {
            // 章节列表数据
            for (var i = start; i < end; i++) {
                _aindex = chapters[i].index;
                _vip = chapters[i].vip;
                _aid = chapters[i].article_id;
                
                _buystat = '';
                if (_vip == 1) {
                    // 判断是否购买
                    isbuy = msxfAPI.readBuyRecord(bid, parseInt(_aid));
                    if (isbuy) {
                        _buystat = '<span class="vip">已购买</span>';
                    } else {
                        _buystat = '<span class="vip">收费</span>';
                    }
                }

                _artstat = _vip == 0 ? '<span class="free">免费</span>' : _buystat;

                li = document.createElement("li");
                li.addClass("item").attr({"id": "aindex_"+_aindex, "data-aindex": _aindex, "data-aid": _aid, "data-vip": _vip});
                li.html(_artstat+_aindex+'.'+chapters[i].title);

                dirlist.insertChild(li);
            }
            myScroll.refresh();
            
            // 重新定位
            var curr_aindex = parseInt(aindex - 1);
            if (curr_aindex >= 0 && aindex) {
                var currObj = $("#aindex_"+aindex);
                if (currObj.id) { 
                    currObj.setClass("act");
                    myScroll.scrollToElement("li:nth-child("+curr_aindex+")", 0);
                } else {
                    // 回到最上面
                    if (Math.abs(myScroll.y) > 100) {
                        myScroll.scrollTo(0, 52, 0);
                    }
                }
            }
        } else {
            // 提示，点击重新加载
            var li = document.createElement("li");
            li.html('章节列表读取失败，点击重新读取章节');
            dirlist.appendChild(li);

            li.bind(CLICK_EVENT, function(e) {
                showMsg("章节列表正在载入中...");
                cacheChapterDir(bid, 0, function() {
                    openDir(aindex);   
                });
            });
        }
    }

    // 点击阅读章节
    function onTouchEvent(e) {
        if (!e) return false;

        var target = e.target;

        // 判断点击的节点，如果是点击子节点，就向上查到根
        var chktarget = target.attr("data-aindex");
        if (!chktarget) {
            var target = target.$p(".item");
            if (!target) return false;            
        }

        var aindex = target.attr("data-aindex");
        if (!aindex) return false;

        // 判断章节是否要付费
        var vip = target.attr("data-vip");
        if (vip == 1) {
            // 判断是否购买过
            var isbuy = msxfAPI.readBuyRecord(bid, parseInt(target.attr("data-aid")));
            if (isbuy) {
                openRead(bid, aindex, 1);
                return;
            }

            // 判断是否登陆
            if (!SID()) {
                showLoginDialog();
                return false;
            }

            showOrderDialog(bid, aindex);
        } else {
            var page = parseInt(nlg(target.attr("data-page"), 1));
            openRead(bid, aindex, page);
        }
    }

})();
